@@ -6,12 +6,15 @@ import pytest
 
 from boutdata.collect import collect
 from boutdata.squashoutput import squashoutput
+from boututils.datafile import DataFile
 
 from boutdata.tests.make_test_data import (
     apply_slices,
     create_dump_file,
     concatenate_data,
     expected_attributes,
+    expected_file_attributes,
+    make_grid_info,
     remove_xboundaries,
     remove_yboundaries,
     remove_yboundaries_upper_divertor,
@@ -56,7 +59,7 @@ def check_collected_data(
         Arrays should be global (not per-process).
     fieldperp_global_yind : int
         Global y-index where FieldPerps are expected to be defined.
-    path : pathlib.Path or str
+    path : pathlib.Path
         Path to collect data from.
     squash : bool
         If True, call `squashoutput()` and delete the `BOUT.dmp.*.nc` files (so that we
@@ -95,115 +98,74 @@ def check_collected_data(
 
     for varname in expected:
         actual = collect(varname, path=path, **collect_kwargs)
-        npt.assert_array_equal(expected[varname], actual)
-        actual_keys = list(actual.attributes.keys())
-        if varname in expected_attributes:
-            for a in expected_attributes[varname]:
-                assert actual.attributes[a] == expected_attributes[varname][a]
-                actual_keys.remove(a)
+        check_variable(
+            varname,
+            actual,
+            expected[varname],
+            expected_attributes.get(varname, None),
+            fieldperp_global_yind,
+        )
 
-        if "fieldperp" in varname:
-            assert actual.attributes["yindex_global"] == fieldperp_global_yind
-            actual_keys.remove("yindex_global")
+    if squash:
+        filename = path.joinpath("boutdata.nc")
+    else:
+        filename = path.joinpath("BOUT.dmp.0.nc")
+    with DataFile(str(filename)) as f:
+        for attrname, attr in expected_file_attributes.items():
+            assert f.read_file_attribute(attrname) == attr
 
-        assert actual_keys == ["bout_type"]
 
-        if "field3d_t" in varname:
-            assert actual.attributes["bout_type"] == "Field3D_t"
-        elif "field3d" in varname:
-            assert actual.attributes["bout_type"] == "Field3D"
-        elif "field2d_t" in varname:
-            assert actual.attributes["bout_type"] == "Field2D_t"
-        elif "field2d" in varname:
-            assert actual.attributes["bout_type"] == "Field2D"
-        elif "fieldperp_t" in varname:
-            assert actual.attributes["bout_type"] == "FieldPerp_t"
-        elif "fieldperp" in varname:
-            assert actual.attributes["bout_type"] == "FieldPerp"
-        elif "_t" in varname or varname == "t_array":
-            assert actual.attributes["bout_type"] == "scalar_t"
-        else:
-            assert actual.attributes["bout_type"] == "scalar"
+def check_variable(
+    varname, actual, expected_data, expected_attributes, fieldperp_global_yind
+):
+    """
+    Check a single variable
+
+    Parameters
+    ----------
+    varname : str
+        Name of the variable
+    actual : BoutArray
+        The collected variable
+    expected_data : np.Array
+        Expected data for the variable
+    expected_attributes : dict or None
+        Expected attributes for the variable
+    fieldperp_global_yind : int
+        Global y-index where FieldPerps have been created
+    """
+    npt.assert_array_equal(expected_data, actual)
+    actual_keys = list(actual.attributes.keys())
+    if expected_attributes is not None:
+        for a in expected_attributes:
+            assert actual.attributes[a] == expected_attributes[a]
+            actual_keys.remove(a)
+
+    if "fieldperp" in varname:
+        assert actual.attributes["yindex_global"] == fieldperp_global_yind
+        actual_keys.remove("yindex_global")
+
+    assert actual_keys == ["bout_type"]
+
+    if "field3d_t" in varname:
+        assert actual.attributes["bout_type"] == "Field3D_t"
+    elif "field3d" in varname:
+        assert actual.attributes["bout_type"] == "Field3D"
+    elif "field2d_t" in varname:
+        assert actual.attributes["bout_type"] == "Field2D_t"
+    elif "field2d" in varname:
+        assert actual.attributes["bout_type"] == "Field2D"
+    elif "fieldperp_t" in varname:
+        assert actual.attributes["bout_type"] == "FieldPerp_t"
+    elif "fieldperp" in varname:
+        assert actual.attributes["bout_type"] == "FieldPerp"
+    elif "_t" in varname or varname == "t_array":
+        assert actual.attributes["bout_type"] == "scalar_t"
+    else:
+        assert actual.attributes["bout_type"] == "scalar"
 
 
 class TestCollect:
-    def make_grid_info(
-        self, *, mxg=2, myg=2, nxpe=1, nype=1, ixseps1=None, ixseps2=None, xpoints=0
-    ):
-        """
-        Create a dict of parameters used for creating test data
-
-        Parameters
-        ----------
-        mxg : int, optional
-            Number of guard cells in the x-direction
-        myg : int, optional
-            Number of guard cells in the y-direction
-        nxpe : int, optional
-            Number of processes in the x-direction
-        nype : int, optional
-            Number of processes in the y-direction
-        ixseps1 : int, optional
-            x-index (where indexing includes boundary points) of point just outside
-            first separatrix
-        ixseps2 : int, optional
-            x-index (where indexing includes boundary points) of point just outside
-            second separatrix
-        xpoints : int, optional
-            Number of X-points.
-        """
-        grid_info = {}
-        grid_info["iteration"] = 6
-        grid_info["MXSUB"] = 3
-        grid_info["MYSUB"] = 4
-        grid_info["MZSUB"] = 5
-        grid_info["MXG"] = mxg
-        grid_info["MYG"] = myg
-        grid_info["MZG"] = 0
-        grid_info["NXPE"] = nxpe
-        grid_info["NYPE"] = nype
-        grid_info["NZPE"] = 1
-        grid_info["nx"] = nxpe * grid_info["MXSUB"] + 2 * mxg
-        grid_info["ny"] = nype * grid_info["MYSUB"]
-        grid_info["nz"] = grid_info["NZPE"] * grid_info["MZSUB"]
-        grid_info["MZ"] = grid_info["nz"]
-        if ixseps1 is None:
-            grid_info["ixseps1"] = grid_info["nx"]
-        else:
-            grid_info["ixseps1"] = ixseps1
-        if ixseps2 is None:
-            grid_info["ixseps2"] = grid_info["nx"]
-        else:
-            grid_info["ixseps2"] = ixseps2
-        if xpoints == 0:
-            grid_info["jyseps1_1"] = -1
-            grid_info["jyseps2_1"] = grid_info["ny"] // 2 - 1
-            grid_info["ny_inner"] = grid_info["ny"] // 2
-            grid_info["jyseps1_2"] = grid_info["ny"] // 2 - 1
-            grid_info["jyseps2_2"] = grid_info["ny"]
-        elif xpoints == 1:
-            if nype < 3:
-                raise ValueError(f"nype={nype} not enough for single-null")
-            yproc_per_region = nype // 3
-            grid_info["jyseps1_1"] = yproc_per_region * grid_info["MYSUB"] - 1
-            grid_info["jyseps2_1"] = grid_info["ny"] // 2 - 1
-            grid_info["ny_inner"] = grid_info["ny"] // 2
-            grid_info["jyseps1_2"] = grid_info["ny"] // 2 - 1
-            grid_info["jyseps2_2"] = 2 * yproc_per_region * grid_info["MYSUB"] - 1
-        elif xpoints == 2:
-            if nype < 6:
-                raise ValueError(f"nype={nype} not enough for single-null")
-            yproc_per_region = nype // 6
-            grid_info["jyseps1_1"] = yproc_per_region * grid_info["MYSUB"] - 1
-            grid_info["jyseps2_1"] = 2 * yproc_per_region * grid_info["MYSUB"] - 1
-            grid_info["ny_inner"] = 3 * yproc_per_region * grid_info["MYSUB"]
-            grid_info["jyseps1_2"] = 4 * yproc_per_region * grid_info["MYSUB"] - 1
-            grid_info["jyseps2_2"] = 5 * yproc_per_region * grid_info["MYSUB"] - 1
-        else:
-            raise ValueError(f"Unsupported value for xpoints: {xpoints}")
-
-        return grid_info
-
     @pytest.mark.parametrize("squash_params", squash_params_list)
     @pytest.mark.parametrize("collect_kwargs", collect_kwargs_list)
     def test_core_min_files(self, tmp_path, squash_params, collect_kwargs):
@@ -212,7 +174,7 @@ class TestCollect:
         """
         squash, squash_kwargs = squash_params
 
-        grid_info = self.make_grid_info()
+        grid_info = make_grid_info()
 
         fieldperp_global_yind = 3
         fieldperp_yproc_ind = 0
@@ -252,6 +214,212 @@ class TestCollect:
             squash_kwargs=squash_kwargs,
         )
 
+    @pytest.mark.parametrize(
+        "time_split",
+        [
+            (None, None),
+            (1, None),
+            (2, None),
+            (2, 3),
+            (3, None),
+            (4, None),
+            (5, None),
+            (6, None),
+            (7, None),
+        ],
+    )
+    def test_core_min_files_existing_squash_file_raises(self, tmp_path, time_split):
+        """
+        Check output from a core-only case using the minimum number of processes
+        """
+        time_split_size, time_split_first_label = time_split
+
+        squash_kwargs = {}
+        if time_split_size is not None:
+            squash_kwargs["time_split_size"] = time_split_size
+        if time_split_first_label is not None:
+            squash_kwargs["time_split_first_label"] = time_split_first_label
+
+        grid_info = make_grid_info()
+
+        fieldperp_global_yind = 3
+
+        rng = np.random.default_rng(100)
+
+        # core
+        # core includes "ylower" and "yupper" even though there is no actual y-boundary
+        # because collect/squashoutput collect these points
+        dump_params = [
+            (0, ["xinner", "xouter", "ylower", "yupper"], fieldperp_global_yind),
+        ]
+        for i, boundaries, fieldperp_yind in dump_params:
+            create_dump_file(
+                tmpdir=tmp_path,
+                rng=rng,
+                grid_info=grid_info,
+                i=i,
+                boundaries=boundaries,
+                fieldperp_global_yind=fieldperp_yind,
+            )
+
+        if time_split_size is None:
+            filenames = ["boutdata.nc"]
+        else:
+            first = 0 if time_split_first_label is None else time_split_first_label
+            # Assumes 'nt' is always 6, as set by make_grid_info() and
+            # create_dump_file()
+            n_output = (6 + time_split_size - 1) // time_split_size
+            filenames = [f"boutdata{i}.nc" for i in range(first, first + n_output)]
+
+        for f in filenames:
+            # Create file named f
+            existing_file = tmp_path.joinpath(f)
+            existing_file.touch()
+
+            with pytest.raises(
+                ValueError,
+                match=r" will not overwrite. Also, for some filenames collect may try "
+                r"to read from this file, which is presumably not desired behaviour.",
+            ):
+                squashoutput(tmp_path, outputname="boutdata.nc", **squash_kwargs)
+
+            # Remove 'existing_file' so we can check the next one
+            existing_file.unlink()
+
+    @pytest.mark.parametrize(
+        "time_split",
+        [
+            (1, None),
+            (2, None),
+            (2, 3),
+            (3, None),
+            (4, None),
+            (5, None),
+            (6, None),
+            (7, None),
+        ],
+    )
+    @pytest.mark.parametrize("parallel", [False, 2])
+    def test_core_min_files_time_split(self, tmp_path, time_split, parallel):
+        """
+        Check output from a core-only case using the minimum number of processes
+        """
+        collect_kwargs = {"xguards": True, "yguards": "include_upper"}
+        squash_kwargs = {"time_split_size": time_split[0], "parallel": parallel}
+        if time_split[1] is not None:
+            squash_kwargs["time_split_first_label"] = time_split[1]
+
+        grid_info = make_grid_info()
+
+        fieldperp_global_yind = 3
+        fieldperp_yproc_ind = 0
+
+        rng = np.random.default_rng(100)
+
+        # core
+        # core includes "ylower" and "yupper" even though there is no actual y-boundary
+        # because collect/squashoutput collect these points
+        dump_params = [
+            (0, ["xinner", "xouter", "ylower", "yupper"], fieldperp_global_yind),
+        ]
+        dumps = []
+        for i, boundaries, fieldperp_yind in dump_params:
+            dumps.append(
+                create_dump_file(
+                    tmpdir=tmp_path,
+                    rng=rng,
+                    grid_info=grid_info,
+                    i=i,
+                    boundaries=boundaries,
+                    fieldperp_global_yind=fieldperp_yind,
+                )
+            )
+
+        expected = concatenate_data(
+            dumps, nxpe=grid_info["NXPE"], fieldperp_yproc_ind=fieldperp_yproc_ind
+        )
+
+        # Copy of check_collected_data code, modified to test series of output
+        # files created when setting time_split_size
+        ######################################################################
+
+        # Always squash
+        squashoutput(
+            tmp_path, outputname="boutdata.nc", **collect_kwargs, **squash_kwargs
+        )
+        collect_prefix = "boutdata"
+        # Delete dump files to be sure we do not read from them
+        dump_names = glob(str(tmp_path.joinpath("BOUT.dmp.*.nc")))
+        for x in dump_names:
+            Path(x).unlink()
+
+        # Assumes 'nt' is always 6, as set by make_grid_info() and create_dump_file()
+        n_output = (6 + time_split[0] - 1) // time_split[0]
+        if time_split[1] is None:
+            start_ind = 0
+        else:
+            start_ind = time_split[1]
+
+        for i in range(n_output):
+            for varname in expected:
+                actual = collect(
+                    varname,
+                    prefix=collect_prefix + str(start_ind + i),
+                    path=tmp_path,
+                    **collect_kwargs,
+                )
+
+                if "_t" in actual.attributes["bout_type"]:
+                    expected_data = expected[varname][
+                        i * time_split[0] : (i + 1) * time_split[0]
+                    ]
+                else:
+                    expected_data = expected[varname]
+
+                check_variable(
+                    varname,
+                    actual,
+                    expected_data,
+                    expected_attributes.get(varname, None),
+                    fieldperp_global_yind,
+                )
+
+    def test_core_min_files_append_time_split_raises(self, tmp_path):
+        """
+        Check output from a core-only case using the minimum number of processes
+        """
+        collect_kwargs = {"xguards": True, "yguards": "include_upper"}
+        squash_kwargs = {"time_split_size": 2, "append": True}
+
+        grid_info = make_grid_info()
+
+        fieldperp_global_yind = 3
+
+        rng = np.random.default_rng(100)
+
+        # core
+        # core includes "ylower" and "yupper" even though there is no actual y-boundary
+        # because collect/squashoutput collect these points
+        dump_params = [
+            (0, ["xinner", "xouter", "ylower", "yupper"], fieldperp_global_yind),
+        ]
+        for i, boundaries, fieldperp_yind in dump_params:
+            create_dump_file(
+                tmpdir=tmp_path,
+                rng=rng,
+                grid_info=grid_info,
+                i=i,
+                boundaries=boundaries,
+                fieldperp_global_yind=fieldperp_yind,
+            )
+
+        with pytest.raises(
+            ValueError, match="'time_split_size' is not compatible with append=True"
+        ):
+            squashoutput(
+                tmp_path, outputname="boutdata.nc", **collect_kwargs, **squash_kwargs
+            )
+
     @pytest.mark.parametrize("squash_params", squash_params_list)
     @pytest.mark.parametrize("collect_kwargs", collect_kwargs_list)
     def test_core(self, tmp_path, squash_params, collect_kwargs):
@@ -262,7 +430,7 @@ class TestCollect:
         """
         squash, squash_kwargs = squash_params
 
-        grid_info = self.make_grid_info(nxpe=3, nype=3)
+        grid_info = make_grid_info(nxpe=3, nype=3)
 
         fieldperp_global_yind = 3
         fieldperp_yproc_ind = 0
@@ -318,7 +486,7 @@ class TestCollect:
         """
         squash, squash_kwargs = squash_params
 
-        grid_info = self.make_grid_info(ixseps1=0, ixseps2=0)
+        grid_info = make_grid_info(ixseps1=0, ixseps2=0)
 
         fieldperp_global_yind = 3
         fieldperp_yproc_ind = 0
@@ -366,7 +534,7 @@ class TestCollect:
         """
         squash, squash_kwargs = squash_params
 
-        grid_info = self.make_grid_info(nxpe=3, nype=3, ixseps1=0, ixseps2=0)
+        grid_info = make_grid_info(nxpe=3, nype=3, ixseps1=0, ixseps2=0)
 
         fieldperp_global_yind = 3
         fieldperp_yproc_ind = 0
@@ -420,7 +588,7 @@ class TestCollect:
         """
         squash, squash_kwargs = squash_params
 
-        grid_info = self.make_grid_info(nype=3, ixseps1=4, xpoints=1)
+        grid_info = make_grid_info(nype=3, ixseps1=4, xpoints=1)
 
         fieldperp_global_yind = 7
         fieldperp_yproc_ind = 1
@@ -473,7 +641,7 @@ class TestCollect:
         """
         squash, squash_kwargs = squash_params
 
-        grid_info = self.make_grid_info(nype=3, ixseps1=4, xpoints=1)
+        grid_info = make_grid_info(nype=3, ixseps1=4, xpoints=1)
 
         fieldperp_global_yind = 1
         fieldperp_yproc_ind = 0
@@ -526,7 +694,7 @@ class TestCollect:
         """
         squash, squash_kwargs = squash_params
 
-        grid_info = self.make_grid_info(nype=3, ixseps1=4, xpoints=1)
+        grid_info = make_grid_info(nype=3, ixseps1=4, xpoints=1)
 
         fieldperp_global_yind = 14
         fieldperp_yproc_ind = 2
@@ -581,7 +749,7 @@ class TestCollect:
 
         collect_kwargs = {"xguards": True, "yguards": "include_upper"}
 
-        grid_info = self.make_grid_info(nype=3, ixseps1=4, xpoints=1)
+        grid_info = make_grid_info(nype=3, ixseps1=4, xpoints=1)
 
         fieldperp_global_yind = 7
         fieldperp_yproc_ind = 1
@@ -637,7 +805,7 @@ class TestCollect:
 
         collect_kwargs = {"xguards": True, "yguards": "include_upper"}
 
-        grid_info = self.make_grid_info(nype=3, ixseps1=4, xpoints=1)
+        grid_info = make_grid_info(nype=3, ixseps1=4, xpoints=1)
 
         fieldperp_global_yind = 7
         fieldperp_yproc_ind = 1
@@ -690,7 +858,7 @@ class TestCollect:
         """
         squash, squash_kwargs = squash_params
 
-        grid_info = self.make_grid_info(nxpe=3, nype=9, ixseps1=7, xpoints=1)
+        grid_info = make_grid_info(nxpe=3, nype=9, ixseps1=7, xpoints=1)
 
         fieldperp_global_yind = 19
         fieldperp_yproc_ind = 4
@@ -777,7 +945,7 @@ class TestCollect:
         means there is at least one process in each region with no edges touching
         another region.
         """
-        grid_info = self.make_grid_info(nxpe=3, nype=9, ixseps1=7, xpoints=1)
+        grid_info = make_grid_info(nxpe=3, nype=9, ixseps1=7, xpoints=1)
 
         fieldperp_global_yind = 19
         fieldperp_yproc_ind = 4
@@ -1212,7 +1380,7 @@ class TestCollect:
             "zind": zind,
         }
 
-        grid_info = self.make_grid_info(nxpe=3, nype=9, ixseps1=7, xpoints=1)
+        grid_info = make_grid_info(nxpe=3, nype=9, ixseps1=7, xpoints=1)
 
         fieldperp_global_yind = 19
         fieldperp_yproc_ind = 4
@@ -1294,7 +1462,7 @@ class TestCollect:
         """
         squash, squash_kwargs = squash_params
 
-        grid_info = self.make_grid_info(nype=6, ixseps1=4, ixseps2=4, xpoints=2)
+        grid_info = make_grid_info(nype=6, ixseps1=4, ixseps2=4, xpoints=2)
 
         fieldperp_global_yind = 7
         fieldperp_yproc_ind = 1
@@ -1352,9 +1520,7 @@ class TestCollect:
         """
         squash, squash_kwargs = squash_params
 
-        grid_info = self.make_grid_info(
-            nxpe=3, nype=18, ixseps1=7, ixseps2=7, xpoints=2
-        )
+        grid_info = make_grid_info(nxpe=3, nype=18, ixseps1=7, ixseps2=7, xpoints=2)
 
         fieldperp_global_yind = 19
         fieldperp_yproc_ind = 4
@@ -1461,7 +1627,7 @@ class TestCollect:
         """
         squash, squash_kwargs = squash_params
 
-        grid_info = self.make_grid_info(nype=6, ixseps1=3, ixseps2=5, xpoints=2)
+        grid_info = make_grid_info(nype=6, ixseps1=3, ixseps2=5, xpoints=2)
 
         fieldperp_global_yind = 7
         fieldperp_yproc_ind = 1
@@ -1523,7 +1689,7 @@ class TestCollect:
         """
         squash, squash_kwargs = squash_params
 
-        grid_info = self.make_grid_info(
+        grid_info = make_grid_info(
             mxg=mxg, myg=myg, nxpe=3, nype=18, ixseps1=6, ixseps2=11, xpoints=2
         )
 
@@ -1643,9 +1809,7 @@ class TestCollect:
         edges touching another region. This test checks some compression options that
         can be used with `squashoutput()`, verifying that they do not modify data.
         """
-        grid_info = self.make_grid_info(
-            nxpe=3, nype=18, ixseps1=6, ixseps2=11, xpoints=2
-        )
+        grid_info = make_grid_info(nxpe=3, nype=18, ixseps1=6, ixseps2=11, xpoints=2)
 
         fieldperp_global_yind = 19
         fieldperp_yproc_ind = 4
